#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import QWidget, QHBoxLayout, QLabel, QFrame, QApplication
from models.enums import setLongTextToElideFormat,BUTTON_PRESSED,BUTTON_LEAVE,BUTTON_ENTER
from ui.systembutton import *
from models.enums import UBUNTUKYLIN_RES_WIN_PATH,UBUNTUKYLIN_HTTP_WIN_RES_PATH,ITEM_LABEL_STYLE,AppMsg
from models.application import *
from utils import commontools

class Win_Replace_Card(QFrame):
    clicked_card = pyqtSignal(Application)
    def __init__(self,detail,win_msg,app_msg,parent = None):
        QFrame.__init__(self,parent)
        self.win_msg = win_msg
        self.app = app_msg
        self.detail_widget = detail
        self.m_parent = parent
        self.init_ui()
        self.init_style()
        self.clicked_card.connect(self.detail_widget.slot_clicked_card)
        self.show()

    def init_ui(self):
        self.resize(413,114)
        self.setStyleSheet("QFrame{background-color: #e6e6e6;border:0px solid #ff0000;border-radius:10px;}")

        self.icon1_label = QLabel(self)
        self.icon1_label.resize(40,40)
        self.icon1_label.move(20,29)
        self.appname1_label = QLabel(self)
        self.appname1_label.resize(84,20)
        self.appname1_label.setStyleSheet("QLabel{border:0px;font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;}")
        self.appname1_label.move(74,25)
        self.category1_label = QLabel(self)
        self.category1_label.resize(24,17)
        self.category1_label.setStyleSheet("QLabel{border:0px;font-size: 12px;font-weight: 400;color: #8F9399;line-height: 17px;}")
        self.category1_label.move(74,59)
        self.category1_label.show()

        self.arrowhead_label = QLabel(self)
        self.arrowhead_label.resize(50,31)
        self.arrowhead_label.setStyleSheet("QLabel{border:0px;background-image:url(./res/arrowhead.png);background-color:transparent;}")
        self.arrowhead_label.move((self.width()-self.arrowhead_label.width())/2 ,(self.height()-self.arrowhead_label.height())/2)
        self.arrowhead_label.show()

        self.icon2_label = QLabel(self)
        self.icon2_label.resize(40, 40)
        self.icon2_label.move(247, 29)
        self.appname2_label = QLabel(self)
        self.appname2_label.resize(84, 20)
        self.appname2_label.setStyleSheet("QLabel{border:0px;font-size: 14px;font-weight: 400;color: #303133;line-height: 20px;}")
        self.appname2_label.move(301, 25)
        self.category2_label = QLabel(self)
        self.category2_label.resize(24, 17)
        self.category2_label.setStyleSheet("QLabel{border:0px;font-size: 12px;font-weight: 400;color: #8F9399;line-height: 17px;}")
        self.category2_label.move(301, 59)
        self.category2_label.show()

    #
    #函数：初始化界面
    #
    def init_style(self):
        if (os.path.isfile(UBUNTUKYLIN_HTTP_WIN_RES_PATH + str(self.win_msg[4]) + ".png")):
            self.icon1_label.setStyleSheet(
                ITEM_LABEL_STYLE % (UBUNTUKYLIN_HTTP_WIN_RES_PATH + str(self.win_msg[4]) + ".png"))
        elif (os.path.isfile(UBUNTUKYLIN_HTTP_WIN_RES_PATH + str(self.win_msg[4]) + ".jpg")):
            self.icon1_label.setStyleSheet(
                ITEM_LABEL_STYLE % (UBUNTUKYLIN_HTTP_WIN_RES_PATH + str(self.win_msg[4]) + ".jpg"))
        elif (os.path.isfile(UBUNTUKYLIN_RES_WIN_PATH + str(self.win_msg[4]) + ".png")):
            self.icon1_label.setStyleSheet(
                ITEM_LABEL_STYLE % (UBUNTUKYLIN_RES_WIN_PATH + str(self.win_msg[4]) + ".png"))
        elif (os.path.isfile(UBUNTUKYLIN_RES_WIN_PATH + str(self.win_msg[4]) + ".jpg")):
            self.icon1_label.setStyleSheet(
                ITEM_LABEL_STYLE % (UBUNTUKYLIN_RES_WIN_PATH + str(self.win_msg[4]) + ".jpg"))
        else:
            self.icon1_label.setStyleSheet(ITEM_LABEL_STYLE % (UBUNTUKYLIN_RES_WIN_PATH + "default.png"))
        self.icon1_label.show()
        if(self.win_msg[4]!=""):
            self.appname1_label.setText(self.win_msg[4])
        else:
            self.appname1_label.setText(self.win_msg[3])
        self.appname1_label.adjustSize()
        self.appname1_label.show()
        if self.app.category_name_cn != '' and self.app.category_name_cn is not None and self.app.category_name_cn != 'None':
            self.category1_label.setText(self.app.category_name_cn)
            self.category2_label.setText(self.app.category_name_cn)
        else:
            self.category2_label.setText(self.app.category_name)
            self.category1_label.setText(self.app.category_name)
        self.category1_label.adjustSize()
        self.category2_label.adjustSize()

        iconpath = commontools.get_icon_path(self.app.pkgname)
        self.icon2_label.setStyleSheet("QLabel{background-image:url('" + iconpath + "');background-color:transparent;}")
        self.icon2_label.show()
        if self.app.displayname_cn != '' and self.app.displayname_cn is not None and self.app.displayname_cn != 'None':
            self.appname2_label.setText(self.app.displayname_cn)
            self.appname2_label.adjustSize()
        else:
            self.appname2_label.setText(self.app.pkgname)
            self.appname2_label.adjustSize()

    # 鼠标进入事件
    def enterEvent(self, event):
        self.status = BUTTON_ENTER
        self.setGeometry(self.x(),self.y(), self.width() + 5, self.height() + 5)
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setOffset(0, 0)
        self.shadow.setColor(Qt.black)
        self.shadow.setBlurRadius(10)
        self.setGraphicsEffect(self.shadow)
    #
    # 函数：鼠标离开事件
    #

    def leaveEvent(self, event):
        self.status = BUTTON_LEAVE
        self.resize(413,114)
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(0)
        self.shadow.setOffset(0, 0)
        self.setGraphicsEffect(self.shadow)

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        self.clicked_card.emit(self.app)
        if event.button() == Qt.LeftButton:
            self.status = BUTTON_PRESSED
        else:
            event.ignore()



import os
import math
from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from ui.systembutton import *
from ui.show_ratings import *
from models.enums import CACHE_DIR,BUTTON_PRESSED,BUTTON_ENTER,BUTTON_LEAVE,BUTTON_DISABLE
from utils import commontools
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Comment_Pop_Up(QWidget):
    change_background_image = pyqtSignal()
    submit_my_rating_apk = pyqtSignal(int, ApkInfo)
    submit_my_rating_app = pyqtSignal(int, Application)
    def __init__(self,app_msg,my_rating,app_detail,parent = None):
        QWidget.__init__(self, parent)
        self.mouse_press = False
        self.appmsg = app_msg
        self.app_detail = app_detail
        self.MY_SCORE = my_rating
        self.star_list = []
        self.app_name = ""
        self.pre_app_name = ""
        self.init_ui()
        self.paintEvent = self.set_paintEvent
        self.submit_my_rating_apk.connect(self.app_detail.m_parent.mw.slot_submit_my_rating)
        self.submit_my_rating_app.connect(self.app_detail.m_parent.mw.slot_submit_my_rating)
        self.timer = QTimer(self)
        self.timer.timeout.connect(self.slot_timer_timeout)

    def init_ui(self):
        self.setFixedSize(442,317)
        self.setWindowFlags(Qt.FramelessWindowHint | Qt.Tool)
        self.setWindowTitle(_("Comment"))
        self.setAttribute(Qt.WA_TranslucentBackground)
        self.widget = QWidget(self)
        self.widget.resize(424,299)
        self.widget.move(9,9)

        self.title_label = QLabel(self.widget)
        self.title_label.setText(_("Comment"))
        if(Globals.THEME_COLOR == 0):
            self.title_label.setStyleSheet("QLabel{font-size: 14px;color: #1B1B1B;font-weight: 500;line-height: 22px}")
        else:
            self.title_label.setStyleSheet("QLabel{font-size: 14px;color: #8F9399;font-weight: 500;line-height: 14px;background:transparent;}")
        self.title_label.move(16,8)

        self.close_btn = QPushButton(self.widget)
        self.close_btn.setFocusPolicy(Qt.NoFocus)
        self.close_btn.resize(30, 30)
        self.close_btn.setStyleSheet(
            "QPushButton{border-radius: 4px;border:0px;border-image:url('res/close1.png')}QPushButton:hover{border:0px;border-image:url('res/close2.png')}QPushButton:pressed{border:0px;border-image:url('res/close3.png')}")
        self.close_btn.move(self.widget.width() - self.close_btn.width() - 4, 4)
        self.close_btn.setToolTip(_("close"))

        self.line_label = QLabel(self.widget)
        self.line_label.setFixedSize(422, 1)
        self.line_label.setStyleSheet("QLabel{width: 424px;height: 1px;background: #F2F6FD;}")
        self.line_label.move(1, 38)

        self.icon_label = QLabel(self.widget)
        self.icon_label.resize(60,60)
        self.icon_label.move(16,55)
        iconpath = commontools.get_big_icon_path(self.appmsg.pkgname)
        # self.icon_label.setStyleSheet("QLabel{border-radius: 12px;border-image: url('"+iconpath+"') 0 0 0 0 stretch strectch;background-color:transparent;}")
        self.icon_label.setStyleSheet("QLabel{border-radius: 12px;background-color:transparent;}")
        self.pix2 = QPixmap(iconpath)
        self.fitpixmap_userIcon = self.pix2.scaled(60, 60, Qt.IgnoreAspectRatio,
                                                   Qt.SmoothTransformation)
        self.fitpixmap_userIcon = self.PixmapToRound(self.fitpixmap_userIcon, 60, 60)
        self.icon_label.setPixmap(self.fitpixmap_userIcon)
        self.icon_label.setScaledContents(True)

        self.star1 = start_btn(1, "small",self)
        self.star1.move(self.icon_label.x() + self.icon_label.width()+18, self.icon_label.y()+self.widget.y())
        self.star_list.append(self.star1)
        self.star2 = start_btn(2, "small",self)
        self.star2.move(self.star1.x() + self.star1.width() + 10, self.star1.y())
        self.star_list.append(self.star2)
        self.star3 = start_btn(3, "small",self)
        self.star3.move(self.star2.x() + self.star2.width() + 10, self.star1.y())
        self.star_list.append(self.star3)
        self.star4 = start_btn(4, "small",self)
        self.star4.move(self.star3.x() + self.star3.width() + 10, self.star1.y())
        self.star_list.append(self.star4)
        self.star5 = start_btn(5, "small",self)
        self.star5.move(self.star4.x() + self.star4.width() + 10, self.star1.y())
        self.star_list.append(self.star5)

        self.editorial_comments_widget = QFrame(self.widget)
        self.editorial_comments_widget.resize(316, 100)
        self.editorial_comments_widget.move(self.icon_label.x()+self.icon_label.width()+16,self.star1.y()+self.star1.height()+19)
        if (Globals.THEME_COLOR == 0):
            self.editorial_comments_widget.setStyleSheet(
                "QFrame{border-radius: 4px;border: 1px solid #C0C4CC;}QFrame:hover{border-radius: 4px;border: 1px solid #C0C4CC;}")
        else:
            self.editorial_comments_widget.setStyleSheet(
                "QFrame{border-radius: 4px;border: 1px solid #3D3D41;}QFrame:hover{border-radius: 4px;border: 1px solid #3D3D41;}")

        self.submit_btn = QPushButton(self.widget)
        self.submit_btn.resize(64, 30)
        self.submit_btn.setStyleSheet(
            "QPushButton{background: #3790FA;border-radius: 6px;font-size: 14px;font-weight: 400;color: #FFFFFF;text-align: center;line-height: 12px;}QPushButton:hover{background: #40A9FB;;border-radius: 6px;color: #ffffff;}QPushButton:pressed{background: #296CD9;border-radius: 6px;color: #ffffff;}")
        self.submit_btn.setText(_("submit"))
        self.submit_btn.move(self.widget.width() - self.submit_btn.width() - 16,
                             self.editorial_comments_widget.y() + self.editorial_comments_widget.height() + 36)
        self.submit_btn.clicked.connect(self.app_detail.submit_my_review)

        self.comment_edit_label = QTextEdit(self.editorial_comments_widget)
        self.comment_edit_label.resize(314,74)
        self.comment_edit_label.setStyleSheet("QTextEdit{border:0px;font-size: 14px;font-weight: 400;color: #606265;line-height: 20px;}")

        # self.comment_edit_label.verticalScrollBar().setStyleSheet("QScrollBar:vertical{margin:0px 0px 0px 0px;background-color:transparent;border:0px;width:6px;border-radius: 3px;}\
        #                      QScrollBar::handle:vertical{background-color:#C2C2C2;width:6px;border-radius:3px;border:0px;}QScrollBar::handle:vertical:hover{background-color:#8F9399;opacity:0.1;width:6px;border-radius:3px;border:0px;}QScrollBar::handle:vertical:pressed{border:0px;background-color:#8F9399;width:6px;border-radius:3px;}\
        #                      QScrollBar::sub-line:vertical{subcontrol-origin:margin;border:0px solid red;height:13px}\
        #                      QScrollBar::up-arrow:vertical{subcontrol-origin:margin;background-color:rgba(0,0,0,0);height:13px;border:0px}\
        #                      QScrollBar::sub-page:vertical{background-color:transparent;border:0px}\
        #                      QScrollBar::add-page:vertical{background-color:transparent;border:0px}\
        #                      QScrollBar::down-arrow:vertical{background-color:rgba(0,0,0,0);border:0px}\
        #                      QScrollBar::add-line:vertical{subcontrol-origin:margin;border:0px solid green;height:13px}")
        self.comment_edit_label.textChanged.connect(self.slot_comment_edit_label_changed)
        self.comment_edit_label.setText("")
        self.comment_edit_label.move(1,1)

        self.record_comment_length_label = QLabel(self.editorial_comments_widget)
        self.record_comment_length_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 22px;border: 0px;}")
        self.record_comment_length_label.setText("0/200")
        self.record_comment_length_label.adjustSize()
        self.record_comment_length_label.move(self.editorial_comments_widget.width()-self.record_comment_length_label.width()-8,self.editorial_comments_widget.height()-self.record_comment_length_label.height()-4)

        self.tip_label = QLabel(self.widget)
        self.tip_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #F72213;line-height: 24px;}")
        self.tip_label.setText(_("Please rate before commenting"))
        self.tip_label.adjustSize()
        self.tip_label.move(self.editorial_comments_widget.x(),self.editorial_comments_widget.y()+self.editorial_comments_widget.height()+4)
        self.tip_label.hide()

        self.time_tip = QLabel(self.editorial_comments_widget)
        self.time_tip.setStyleSheet(
            "QLabel{background-color:transparent;font-size:14px;font-weight: 400;color: #F56C6C;line-height: 20px;border:none;}")
        self.time_tip.resize(349, 20)

        self.time_tip.setFocusPolicy(Qt.NoFocus)
        self.time_tip.hide()


        self.cancel_btn = QPushButton(self.widget)
        self.cancel_btn.resize(64,30)
        self.cancel_btn.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;border-radius: 6px;text-align: center;border: 1px solid #DDDFE7;}QPushButton:hover{background: #F2F6FD;border-radius: 6px;border: 1px solid #DDDFE7;}QPushButton:pressed{background: #F2F6FD;border-radius: 6px;border: 1px solid #DDDFE7;}")
        self.cancel_btn.setText(_("cancel"))
        self.cancel_btn.move(self.submit_btn.x()-self.cancel_btn.width()-16,self.submit_btn.y())
        if(Globals.LOGIN_SUCCESS):
            self.slot_click_star(self.MY_SCORE,False)
        self.change_star_state()
        self.slot_show_dark_mode()

        self.show()


    def slot_comment_edit_label_changed(self):
        maxLength = 200
        strText = (self.comment_edit_label.toPlainText())
        length = self.comment_edit_label.document().characterCount()
        if(strText == ""):
            if(hasattr(self,"tip_label")):
                self.tip_label.hide()
            self.submit_btn.setEnabled(False)
            self.submit_btn.setStyleSheet("QPushButton{background: #E9E9E9;border-radius: 6px;font-size: 14px;font-weight: 400;color: #B3B3B3;line-height: 20px;}")
            if(hasattr(self,"record_comment_length_label")):
                self.record_comment_length_label.setText("0/200")
                self.record_comment_length_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 22px;border: 0px;}")
                self.record_comment_length_label.adjustSize()
                self.record_comment_length_label.move(
                    self.editorial_comments_widget.width() - self.record_comment_length_label.width() - 8,
                    self.editorial_comments_widget.height() - self.record_comment_length_label.height() - 4)
        else:
            self.submit_btn.setEnabled(True)
            self.submit_btn.setStyleSheet(
                "QPushButton{background: #3790FA;border-radius: 6px;font-size: 14px;font-weight: 400;color: #FFFFFF;text-align: center;line-height: 12px;}QPushButton:hover{background: #40A9FB;;border-radius: 6px;color: #ffffff;}QPushButton:pressed{background: #296CD9;border-radius: 6px;color: #ffffff;}")
            if length-1 > maxLength:

                strText = strText[:-1]
                set_text = strText[0:200]
                self.comment_edit_label.clear()
                self.comment_edit_label.setText(set_text)
                tc = self.comment_edit_label.textCursor()
                tc.movePosition(QTextCursor.End)
                self.comment_edit_label.setTextCursor(tc)
                self.record_comment_length_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #F72213;line-height: 24px;border: 0px;}")
                # self.editorial_comments_widget.setStyleSheet(
                #     "QFrame{border-radius: 12px;border: 2px solid #F56C6C;}QFrame:hover{border-radius: 12px;border: 2px solid #F56C6C;}")
            else:
                self.record_comment_length_label.setText(str(length-1)+"/200")
                self.record_comment_length_label.setStyleSheet(
                    "QLabel{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 22px;border: 0px;}")
                self.record_comment_length_label.adjustSize()
                self.record_comment_length_label.move(
                    self.editorial_comments_widget.width() - self.record_comment_length_label.width() - 8,
                    self.editorial_comments_widget.height() - self.record_comment_length_label.height() - 4)

                if (hasattr(self, "tip_label")):
                    self.tip_label.hide()


    def slot_enter_star(self,num):
        n = 0
        if(num == -1):
            if (Globals.LOGIN_SUCCESS):
                num = self.MY_SCORE
            else:
                num = 0
        for star in self.star_list:
            if(n < num):
                self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_bright_star.png);border:0px;}")
            else:
                self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_dark_star.png);border:0px;}")
            n = n + 1

    def slot_click_star(self,num,need_send = True):
        pre_my_score = self.MY_SCORE
        if (Globals.LOGIN_SUCCESS):
            self.MY_SCORE = num
        else:
            self.MY_SCORE = 0
        #(^-^)
        if(Globals.LOGIN_SUCCESS):
            if(self.app_detail.app_msg.from_ukscdb == True):
                n = 0
                for star in self.star_list:
                    self.star_list[n].mouse_press = True
                    if (n < num):
                        self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_bright_star.png);border:0px;}")
                    else:
                        self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_dark_star.png);border:0px;}")
                    n = n + 1
                #发送评分到服务器 num为多少就是几分
                if(need_send == True):
                    if (self.timer.isActive()):
                        if(self.pre_app_name == self.app_name):
                            self.slot_click_star(pre_my_score, False)
                            self.app_detail.m_parent.mw.show_tip_widget(_("The operation is too frequent. Please try again later!"),True)
                            self.app_detail.m_parent.mw.tip_widget.btn.clicked.connect(self.app_detail.m_parent.mw.slot_close_tipwidget)
                            self.change_star_state()
                        else:
                            if (self.app_detail.btn.text() == _("Open") or self.app_detail.btn.text() == _("installed") or self.app_detail.btn.text() == _("update")):
                                if (type(self.app_detail.app_msg) == ApkInfo):
                                    self.submit_my_rating_apk.emit(num, self.app_detail.app_msg)
                                else:
                                    self.submit_my_rating_app.emit(num, self.app_detail.app_msg)
                                self.timer.start(10000)
                                self.pre_app_name = self.app_name
                            else:
                                # self.refresh_star()
                                self.slot_click_star(self.MY_SCORE, False)
                                self.app_detail.m_parent.mw.show_tip_widget(_("Please install the software before scoring!"), True)
                                self.app_detail.m_parent.mw.tip_widget.btn.clicked.connect(
                                    self.app_detail.m_parent.mw.slot_close_tipwidget)
                                self.change_star_state()
                    else:
                        if(self.app_detail.btn.text() == _("Open") or self.app_detail.btn.text() == _("installed") or self.app_detail.btn.text() == _("update")):
                            if(type(self.app_detail.app_msg) == ApkInfo):
                                self.submit_my_rating_apk.emit(num,self.app_detail.app_msg)
                            else:
                                self.submit_my_rating_app.emit(num, self.app_detail.app_msg)
                            self.timer.start(10000)
                            self.pre_app_name = self.app_name
                        else:
                            # self.refresh_star()
                            self.slot_click_star(pre_my_score, False)
                            # self.slot_click_star(self.MY_SCORE,False)
                            self.app_detail.m_parent.mw.show_tip_widget(_("Please install the software before scoring!"), True)
                            self.app_detail.m_parent.mw.tip_widget.btn.clicked.connect(self.app_detail.m_parent.mw.slot_close_tipwidget)
                            self.change_star_state()
            else:
                if(need_send == True):
                    n = 0
                    for star in self.star_list:
                        self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_dark_star.png);border:0px;}")
                        n = n + 1
                    self.app_detail.m_parent.mw.show_tip_widget(_("Non database software, does not support scoring!"),True)
                    self.app_detail.m_parent.mw.tip_widget.btn.clicked.connect(self.app_detail.m_parent.mw.slot_close_tipwidget)
                    self.change_star_state()
                else:
                    self.refresh_star()
                    self.change_star_state()
        else:
            self.refresh_star()
            self.app_detail.m_parent.mw.show_tip_widget(_("Please log in before scoring!"))
            self.app_detail.m_parent.mw.tip_widget.btn.clicked.connect(self.app_detail.m_parent.mw.slot_close_tipwidget)
            self.change_star_state()

    #
    #函数：刷新星星
    #
    def refresh_star(self):
        n = 0
        for star in self.star_list:
            self.star_list[n].setStyleSheet("QPushButton{background-image: url(./res/medium_dark_star.png);border:0px;}")
            n = n + 1

    def change_star_state(self):
        n = 0
        for star in self.star_list:
            self.star_list[n].mouse_press = False
            n = n + 1

    #
    # 函数：处理计时器超时
    #
    def slot_timer_timeout(self):
        self.timer.stop()
        self.pre_app_name = ""

    def PixmapToRound(self,src,x,y):
        pixmapa = QPixmap(src)
        pixmap = QPixmap(x, y)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing, True)
        # painter.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        path = QPainterPath()
        rect = QRectF(0,0,x,y)
        path.addRoundedRect(rect, 12.0, 12.0)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, x, y, pixmapa)

        return pixmap

    def slot_show_dark_mode(self):
        if (Globals.THEME_COLOR == 0):
            if(Globals.RIGHT_ANGLE):
                self.widget.setStyleSheet(".QWidget{background-color:#ffffff;border: 1px solid rgba(38, 38, 38, 0.15);}")
                self.setStyleSheet("QFrame{background-color:#ffffff;}")
            else:
                self.widget.setStyleSheet(".QWidget{border-radius:12px;background-color:#ffffff;border: 1px solid rgba(38, 38, 38, 0.15);}")
                self.setStyleSheet("QFrame{background-color:#ffffff;}")
            self.title_label.setStyleSheet("QLabel{font-size: 14px;color: #1B1B1B;font-weight: 500;background:transparent;}")
            self.line_label.setStyleSheet("QLabel{width: 424px;height: 1px;background: #F2F6FD;}")
            self.close_btn.setStyleSheet("QPushButton{border-radius: 4px;border:0px;border-image:url('res/close1.png')}QPushButton:hover{border:0px;border-image:url('res/close2.png')}QPushButton:pressed{border:0px;border-image:url('res/close3.png')}")
            self.editorial_comments_widget.setStyleSheet(
                "QFrame{border-radius: 4px;border: 1px solid #C0C4CC;}QFrame:hover{border-radius: 4px;border: 1px solid #C0C4CC;}")
        else:
            if (Globals.RIGHT_ANGLE):
                self.widget.setStyleSheet(".QWidget{background-color:#3D3D41;border: 1px solid rgba(38, 38, 38, 0.15);}")
                self.setStyleSheet("QFrame{background-color:#1F2022;}")
            else:
                self.setStyleSheet("QFrame{background-color:#1F2022;}")
                self.widget.setStyleSheet(".QWidget{border-radius:12px;background-color:#3D3D41;border: 1px solid rgba(38, 38, 38, 0.15);}")
            self.title_label.setStyleSheet("QLabel{font-size: 14px;color: #8F9399;line-height: 14px;font-weight: 500;background:transparent;}")
            self.line_label.setStyleSheet("QLabel{width: 424px;height: 1px;background: #48484C;}")
            self.close_btn.setStyleSheet(
                "QPushButton{border-radius: 4px;border:0px;border-image:url('res/common-close-white.png')}QPushButton:hover{border:0px;border-image:url('res/hover-close.png')}QPushButton:pressed{border:0px;border-image:url('res/clicked-close.png')}")
            self.editorial_comments_widget.setStyleSheet(
                "QFrame{border-radius: 4px;border: 1px solid #3D3D41;}QFrame:hover{border-radius: 4px;border: 1px solid #3D3D41;}")

    #
    # 函数：重绘函数
    #
    def set_paintEvent(self, event):
        painter = QPainter(self)
        m_defaultBackgroundColor = QColor(qRgba(0, 0, 0, 16))
        m_defaultBackgroundColor.setAlpha(50)
        path = QPainterPath()
        path.setFillRule(Qt.WindingFill)
        path.addRoundedRect(10, 10, self.width() - 20, self.height() - 20, 12, 12)

        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.fillPath(path, QBrush(QColor(m_defaultBackgroundColor.red(),
                                             m_defaultBackgroundColor.green(),
                                             m_defaultBackgroundColor.blue())))

        color = QColor(0, 0, 0, 16)
        i = 0
        while i < 4:
            path = QPainterPath()
            path.setFillRule(Qt.WindingFill)
            path.addRoundedRect(10 - i, 10 - i, self.width() - (10 - i) * 2, self.height() - (10 - i) * 2, 12, 12)
            color.setAlpha(int(100 - math.sqrt(i) * 50))
            painter.setPen(color)
            painter.drawPath(path)
            i = i + 1

        painter.setRenderHint(QPainter.Antialiasing)

    #
    # 函数：实现无标题栏窗口拖动
    #
    def mousePressEvent(self, event):
        if (event.button() == Qt.LeftButton):
            self.mouse_press = True
            self.drag_pos = event.globalPos() - self.pos()
            event.accept()

    def mouseReleaseEvent(self, event):
        self.mouse_press = False
        self.setWindowOpacity(1)

    # def mouseMoveEvent(self, event):
    #     if (self.mouse_press):
    #         self.move_pos = event.globalPos()
    #         self.move(self.move_pos - self.drag_pos)
    #         self.setWindowOpacity(1)
    #         event.accept()



from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from ui.systembutton import *
from ui.systembutton import *
from models.enums import CONF_PATH
import datetime
import math
import configparser
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class MenuBar(QWidget):
    click_item = pyqtSignal(QListWidgetItem)
    def __init__(self,width,height,item_num,item_name_list,mw,parent = None):
        QFrame.__init__(self, parent)
        self.w = width
        self.h = height
        self.item_num = item_num
        self.mouse_press = False
        self.item_name_list = item_name_list
        self.setMouseTracking(True)
        self.mw = mw
        self.init_ui()
        self.paintEvent = self.set_paintEvent

    def init_ui(self):
        self.resize(self.w+8+20,self.item_num*(self.h+2)+4+20)
        self.setWindowFlags(Qt.FramelessWindowHint|Qt.Dialog)
        self.setAttribute(Qt.WA_TranslucentBackground)

        self.widget = QFrame(self)
        self.widget.resize(self.w+8,self.item_num*(self.h+2)+4)
        self.widget.move(10,10)

        self.item_list = QListWidget(self.widget)
        self.item_list.setFocusPolicy(Qt.NoFocus)
        self.item_list.resize(self.w,self.item_num*self.h)
        self.item_list.move(4,4)
        self.slot_show_dark_mode()

        self.widget.resize(self.widget.width(),self.item_list.height()+8)
        self.resize(self.width(),self.widget.height()+20)

        for i_name in self.item_name_list:
            item = QListWidgetItem(self.item_list)
            # list_item_widget = menu_list_widget(self.w, self.h, i_name, True, False, self)
            if(self.item_num == 6):
                if(i_name == _("Theme")):
                    list_item_widget = menu_list_widget(self.w,self.h,i_name,True,True,self)
                    list_item_widget.setFocusPolicy(Qt.NoFocus)
                    list_item_widget.clicked_item.connect(self.slot_clicked_item)
                    list_item_widget.enter_signal.connect(self.slot_enter_theme)
                else:
                    list_item_widget = menu_list_widget(self.w, self.h, i_name, True, False,self)
                    list_item_widget.setFocusPolicy(Qt.NoFocus)
                    list_item_widget.clicked_item.connect(self.slot_clicked_item)
                    list_item_widget.leave_signal.connect(self.slot_leave_theme)
                list_item_widget.item_name_label.move(12,int((list_item_widget.height()-list_item_widget.item_name_label.height())/2))
            else:
                cf = configparser.ConfigParser()
                cf.read(CONF_PATH)
                light = cf.get("theme-color", "light")
                dark = cf.get("theme-color", "dark")
                auto = cf.get("theme-color", "auto")
                item_name = "Light mode"
                if (light == "YES"):
                    item_name = _("Light mode")
                if (dark == "YES"):
                    item_name = _("Dark Mode")
                if (auto == "YES"):
                    item_name = _("Automatic")
                if (i_name == item_name):
                    list_item_widget = menu_list_widget(self.w, self.h, i_name, False, True,self)
                    list_item_widget.setFocusPolicy(Qt.NoFocus)
                    list_item_widget.clicked_item.connect(self.slot_clicked_item)
                else:
                    list_item_widget = menu_list_widget(self.w, self.h, i_name, False, False,self)
                    list_item_widget.setFocusPolicy(Qt.NoFocus)
                    list_item_widget.clicked_item.connect(self.slot_clicked_item)

                list_item_widget.item_name_label.move(list_item_widget.icon_label.x() + list_item_widget.icon_label.width() + 11,
                                          int((list_item_widget.height() - list_item_widget.item_name_label.height()) / 2))
            self.item_list.addItem(item)
            self.item_list.setItemWidget(item, list_item_widget)
            self.check_item_color()

    def slot_show_dark_mode(self):
        if(Globals.THEME_COLOR == 0):
            if(Globals.RIGHT_ANGLE):
                self.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#DD0D00;}")
                self.widget.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#FFFFFF;}")
            else:
                self.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#ffffff;border-radius: 8px;}")
                self.widget.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#ffffff;border-radius: 8px;}")
            self.item_list.setStyleSheet("QListWidget{background-color:transparent;border:0px solid #cfcfcf;}QListWidget::item{height:36;background-color:transparent;margin-top:0px;border:0px solid #cfcfcf;border-radius: 6px;}QListWidget::item:hover{background:#3790FA;border-radius: 6px;}")
        else:
            if (Globals.RIGHT_ANGLE):
                self.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#3D3D41;}")
                self.widget.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#3D3D41;}")
            else:
                self.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#3D3D41;border-radius: 8px;}")
                self.widget.setStyleSheet("QFrame{border:0px solid #ff0000;background-color:#3D3D41;border-radius: 8px;}")
            self.item_list.setStyleSheet("QListWidget{background-color:#3D3D41;border:0px solid #cfcfcf;border-radius: 8px;}QListWidget::item{height:36px;background-color:3D3D41;margin-top:0px;border:0px solid #cfcfcf;border-radius: 6px;}QListWidget::item:hover{background:#296CD9;border-radius: 6px;}")
        self.check_item_color()
        # QApplication.processEvents()

    def slot_clicked_item(self,item_name):
        item_count = self.item_list.count()
        for i in range(item_count):
            item = self.item_list.item(i)
            get_item_widget = self.item_list.itemWidget(item)
            if(get_item_widget.item_name_label.text() == item_name):
                self.click_item.emit(item)
                break

        # self.click_item.emit(item_name)

    def slot_enter_theme(self):
        for i in range(self.item_list.count()):
            item = self.item_list.item(i)
            item_widget = self.item_list.itemWidget(item)
            if(item_widget.item_name_label.text() == _("Theme")):
                self.mw.slot_click_first_level(item)
                break

    def slot_leave_theme(self):
        if (hasattr(self.mw, "second_lavel_menubar")):
            self.mw.second_lavel_menubar.hide()

    def check_item_color(self):
        for i in range(self.item_list.count()):
            item = self.item_list.item(i)
            get_item_widget = self.item_list.itemWidget(item)
            if(Globals.THEME_COLOR == 0):
                get_item_widget.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;line-height: 14px;}")
                if(get_item_widget.primary_directory == False):
                    get_item_widget.icon_label.setStyleSheet("QLabel{background-image:url(./res/check2.png);background-color:transparent;}")
            else:
                get_item_widget.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #F9F9F9;line-height: 14px;}")
                if (get_item_widget.primary_directory == False):
                    get_item_widget.icon_label.setStyleSheet("QLabel{background-image:url(./res/check_dark.png);background-color:transparent;}")

    #
    # 函数：重绘函数
    #
    def set_paintEvent(self, event):
        painter = QPainter(self)
        m_defaultBackgroundColor = QColor(qRgb(192, 192, 192))
        m_defaultBackgroundColor.setAlpha(50)
        path = QPainterPath()
        path.setFillRule(Qt.WindingFill)
        path.addRoundedRect(10, 10, self.width() - 20, self.height() - 20, 8, 8)

        painter.setRenderHint(QPainter.Antialiasing, True)
        painter.fillPath(path, QBrush(QColor(m_defaultBackgroundColor.red(),
                                             m_defaultBackgroundColor.green(),
                                             m_defaultBackgroundColor.blue())))

        color = QColor(0, 0, 0, 20)
        i = 0
        while i < 4:
            path = QPainterPath()
            path.setFillRule(Qt.WindingFill)
            path.addRoundedRect(10 - i, 10 - i, self.width() - (10 - i) * 2, self.height() - (10 - i) * 2, 8, 8)
            color.setAlpha(int(100 - math.sqrt(i) * 50))
            painter.setPen(color)
            painter.drawPath(path)
            i = i + 1

        painter.setRenderHint(QPainter.Antialiasing)

    #
    # 函数：鼠标进入事件
    #
    def enterEvent(self,event):
        self.status = BUTTON_ENTER
        self.update()

    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self,event):
        if(event.button() == Qt.LeftButton):
            self.setAttribute(Qt.WA_NoMouseReplay)
            self.mouse_press = True
            self.status = BUTTON_PRESSED
            self.update()

    #
    # 函数：鼠标释放事件
    #
    def mouseReleaseEvent(self,event):
        if(self.mouse_press and self.rect().contains(event.pos())):
            self.mouse_press = False
            self.status = BUTTON_ENTER
            self.update()

    #
    # 函数：鼠标离开事件
    #
    def leaveEvent(self,event):
        self.status = BUTTON_LEAVE
        self.update()


class menu_list_widget(QWidget):
    enter_signal = pyqtSignal()
    leave_signal = pyqtSignal()
    clicked_item = pyqtSignal(str)
    def __init__(self,w,h,item_name,primary_directory,need_icon,parent = None):
        QWidget.__init__(self, parent)
        self.primary_directory = primary_directory
        self.resize(w,h)
        self.item_name_label = QLabel(self)
        self.item_name_label.setAttribute(Qt.WA_TranslucentBackground)
        self.item_name_label.setText(item_name)
        self.item_name_label.adjustSize()
        self.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;line-height: 14px;}")
        self.item_name_label.move(26,int((self.height()-self.item_name_label.height())/2))

        self.icon_label = QLabel(self)
        self.icon_label.resize(12,13)
        if(need_icon == True and primary_directory == True):
            self.icon_label.move(self.width() - self.icon_label.width() - 8,int((self.height() - self.icon_label.height()) / 2))
            self.icon_label.setStyleSheet("QLabel{background-image:url(./res/right_btn.png);background-color:transparent;}")
        elif(need_icon == True and primary_directory == False):
            self.icon_label.move(8,int((self.height() - self.icon_label.height()) / 2))
            self.icon_label.setStyleSheet("QLabel{background-image:url(./res/check2.png);background-color:transparent;}")
        else:
            self.icon_label.move(8, int((self.height() - self.icon_label.height()) / 2))
            self.icon_label.setStyleSheet("QLabel{background-image:url(./res/check2.png);background-color:transparent;}")
            self.icon_label.hide()

    def enterEvent(self,event):
        self.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #FFFFFF;}")
        if (self.item_name_label.text() == _("Theme")):
            self.enter_signal.emit()
        else:
            self.leave_signal.emit()

    def leaveEvent(self, event):
        if(Globals.THEME_COLOR == 0):
            self.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #303133;line-height: 14px;}")
        else:
            self.item_name_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #FFFFFF;line-height: 14px;}")

    def mousePressEvent(self, event):
        # if (event.button() == Qt.LeftButton):
        pass


    def mouseReleaseEvent(self, event):
        self.clicked_item.emit(self.item_name_label.text())



#!/usr/bin/python3
# -*- coding: utf-8 -*
import dbus
import os
import sys
import shutil
import locale
import datetime
import time
import base64
from utils.aescrypt import *
from gi.repository import GObject
import logging
import json

from PyQt5.QtCore import *
from PyQt5 import QtDBus
UBUNTUKYLIN_SOFTWARECENTER_ACTION = 'com.kylin.softwarecenter.action'
from models.enums import (UBUNTUKYLIN_SERVICE_PATH,
                          UBUNTUKYLIN_INTERFACE_PATH,
                          Signals,
                          AppActions,
                          AptProcessMsg,
                          AptActionMsg,
                          ERR_LOG,
                          INFO_LOG,
                          DEBUG_LOG,
                          KYDROID_VERSION_D)

import multiprocessing

from dbus.mainloop.glib import DBusGMainLoop, threads_init
from models.globals import Globals
#from dbus.mainloop.qt import DBusQtMainLoop
#mainloop = DBusQtMainLoop()
from utils.log_manager import LogManager
import gettext
gettext.textdomain("kylin-software-center")
_ = gettext.gettext
# LOG = logging.getLogger("uksc")


class Dbus_Service(QObject,Signals):

    dbus_apt_process = pyqtSignal(str,str,str,int,str)
    copy_finish = pyqtSignal(str,str)
    check_apt_lock_finish = pyqtSignal(str,str,int)

    def __init__(self,need_init_connect):
        QObject.__init__(self)
        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")

        self.iface = None
        self.need_init_connect = need_init_connect

    #
    # 函数：初始化dbus接口
    #
    def init_dbus_ifaces(self):
        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces (" + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f") + ")")
            DBusGMainLoop(set_as_default=True)
            threads_init()
            bus = dbus.SystemBus()
        except Exception as e:
            Globals.LOG_ERROR.logger.error("dbus exception:%s" % str(e))
            #self.init_models_ready.emit("fail","初始化失败!")
            self.init_models_ready.emit("fail", _("Initialization failed"))
            return False

        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces call blocking (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            bus.call_blocking(UBUNTUKYLIN_SERVICE_PATH, '/', UBUNTUKYLIN_INTERFACE_PATH, 'wakeup', None, (), timeout=5)
        except dbus.DBusException as e:
            Globals.LOG_ERROR.logger.error("InstallBackend DBusConnection call blocking exception: %s" % str(e))
            return False

        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces bus.get_object (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            obj = bus.get_object(UBUNTUKYLIN_SERVICE_PATH, '/')
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces dbus.Interface (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            self.iface = dbus.Interface(obj, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces finished (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))

            # lixiang: Repeated starts dbus service may cause 25 seconds stuck because of bus.get_object or connect_to_signal
            #if self.iface is not None:
            #    LOG.info("InstallBackend start to connect signal named software_apt_signal (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            #    self.iface.connect_to_signal("software_apt_signal",self._on_software_apt_signal, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            #    LOG.info("InstallBackend start to connect signal named software_fetch_signal (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            #    self.iface.connect_to_signal("software_fetch_signal",self._on_software_fetch_signal, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            #    LOG.info("InstallBackend start to connect signal named software_auth_signal (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            #    self.iface.connect_to_signal("software_auth_signal",self._on_software_auth_signal, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            #    LOG.info("InstallBackend connect signals finished (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            #else:
            #    LOG.info("InstallBackend dbus interface in none (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
        except dbus.DBusException as e:
#            bus_name = dbus.service.BusName('com.ubuntukylin.softwarecenter', bus)
#            self.dbusControler = SoftwarecenterDbusController(self, bus_name)
#           self.init_models_ready.emit("fail","初始化失败!")
            self.init_models_ready.emit("fail",_("Initialization failed"))
            Globals.LOG_ERROR.logger.error("InstallBackend DBusConnection exception: %s" % str(e))
            return False
        if(self.need_init_connect == True):
            # lixiang: QTimer no response ???
            GObject.timeout_add(2000, self.slotTimeout)

        return True

    #lixiang
    def slotTimeout(self):
        Globals.LOG_INFO.logger.info("InstallBackend slotTimeout Responsed (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
        if self.iface is not None:
            self.iface.connect_to_signal("software_apt_signal", self._on_software_apt_signal, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            self.iface.connect_to_signal("software_fetch_signal", self._on_software_fetch_signal,dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            self.iface.connect_to_signal("software_auth_signal", self._on_software_auth_signal,dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
            self.iface.connect_to_signal("copy_finish",self.call_copy_finish,dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)
        else:
            Globals.LOG_INFO.logger.info("InstallBackend dbus interface in none (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
        return False

    # 安卓环境启动检测dbus接口
    def kydroid_dbus_ifaces(self):
        self.kydroid_iface = None
        try:
            bus = dbus.SystemBus()
        except Exception as e:
            Globals.LOG_ERROR.logger.error("dbus exception:%s" % str(e))
            # self.init_models_ready.emit("fail","安卓dbus初始化失败!")
            self.init_models_ready.emit("fail", _("Android dbus initialization failed"))
            return False
        dbus_server = "cn.kylinos." + KYDROID_VERSION_D
        dbus_path = "/cn/kylinos/" + KYDROID_VERSION_D
        try:
            obj = bus.get_object(dbus_server,
                                 dbus_path,
                                 dbus_server)
            self.kydroid_iface = dbus.Interface(obj, dbus_server)

        #            self.call_dbus_iface("check_source_ubuntukylin")

        # self.iface.connect_to_signal("software_fetch_signal",self._on_software_fetch_signal)
        # self.iface.connect_to_signal("software_apt_signal",self._on_software_apt_signal)
        # self.iface.connect_to_signal("software_auth_signal",self._on_software_auth_signal)
        except dbus.DBusException as e:
            #            bus_name = dbus.service.BusName('com.ubuntukylin.softwarecenter', bus)
            #            self.dbusControler = SoftwarecenterDbusController(self, bus_name)
            #           self.init_models_ready.emit("fail","安卓dbus初始化失败!")
            self.init_models_ready.emit("fail", _("Android dbus initialization failed"))
            Globals.LOG_ERROR.logger.error("dbus exception:%s" % str(e))
            return False

        return True



    #
    # 函数：获取kydroid环境运行状态
    #
    def get_kydroid_evnrun(self,name,uid,prop):
        return self.call_kydroid_dbus_iface("GetPropOfContainer",name,uid,prop)

    #
    #函数：退出dbus后台
    #
    def exit_dbus(self):
        self.iface.exit()

    #
    #函数：进行私钥加密
    #
    def add_key(self,kwargs):
        get_string = base64.b64decode(Globals.STRING).decode()
        res = QByteArray()
        if(kwargs != None):
            for i in kwargs:
                res.append(i)
        else:
            res.append(kwargs)
        #生成一个md5的散列
        cryptographichash = QCryptographicHash(QCryptographicHash.Md5)
        #计算出哈希值
        cryptographichash.addData(res)
        #获取最终的哈希值
        md5 = cryptographichash.result()
        aescryptor = Aescrypt()
        get_string = "-----BEGIN RSA PRIVATE KEY-----\n" + get_string + "\n-----END RSA PRIVATE KEY-----"
        #进行加签
        re_text = aescryptor.add_key_encrypt(get_string,md5.data())  #结果为byte类型
        return re_text.decode()

    # call the dbus functions by function name
    def call_dbus_iface(self, funcname, kwargs=None):
        signature_data = self.add_key(kwargs)
        if self.iface is None:
            return None
        func = getattr(self.iface, funcname)
        if func is None:
            return None

        res = None
        try:
            if(funcname == AppActions.REMOVE or funcname == AppActions.INSTALL):
                res = func(kwargs,signature_data)
            else:
                res = func(kwargs)
        except dbus.DBusException as e:
            self.init_dbus_ifaces()
            try:
                self.call_dbus_iface(funcname)
            except:
                pass
            Globals.LOG_ERROR.logger.error("apt-daemon dbus exception:%s" % str(e))
            return None
        return res

    #
    # 函数：安装本地deb包调用
    #
    def install_debfile(self, path):
        debcache_dir = os.path.join(os.path.expanduser("~"), ".cache", "uksc", "debfile")
        if(os.path.exists(debcache_dir) == False):
            os.makedirs(debcache_dir)
        if(os.path.exists(path)):
            shutil.copy(path, debcache_dir)
        debcache_path = os.path.join(debcache_dir,os.path.split(path)[1])

        return self.call_dbus_iface(AppActions.INSTALLDEBFILE, debcache_path)

    def install_package(self,pkgname):
        return self.call_dbus_iface(AppActions.INSTALL,pkgname)

    # def mk_dir(self,path):
    #     self.iface.mkdir(path)

    def run_app(self,cmd):
        self.iface.run_app(cmd)

    def remove_package(self,pkgname):
        return self.call_dbus_iface(AppActions.REMOVE,pkgname)

    def upgrade_package(self,pkgname):
        return self.call_dbus_iface(AppActions.UPGRADE,pkgname)

    def cancel_package(self,cancelinfo):
        return self.call_dbus_iface(AppActions.CANCEL,cancelinfo)

    def update_source(self,quiet=False):
        return self.call_dbus_iface(AppActions.UPDATE,False)

    def update_source_first_os(self,quiet=False):
        return self.call_dbus_iface(AppActions.UPDATE_FIRST,False)

    def add_source(self,text):
        return self.call_dbus_iface(AppActions.ADD_SOURCE,text)

    def remove_source(self,text):
        return self.call_dbus_iface(AppActions.REMOVE_SOURCE,text)

    def clear_dbus_worklist(self):
        return self.call_dbus_iface("clear_all_work_item")

    def check_dbus_workitem(self):
        return self.call_dbus_iface("check_work_item")

    def check_uksc_is_working(self):
        return self.call_dbus_iface("check_uksc_is_working")

    def set_uksc_not_working(self):
        return self.call_dbus_iface("set_uksc_not_working")

    def exit_uksc_apt_daemon(self):
        self.call_dbus_iface("exit")

    def uksc_exit(self):
        re = self.call_dbus_iface("uksc_exit")

    def call_check_apt_lock(self,appname,action,percent):
        self.call_dbus_iface("check_apt_lock")
        self.check_apt_lock_finish.emit(appname,action,percent)


    def check_dpkg_statu(self):
        self.call_dbus_iface("check_dpkg_statu")

    def call_kydroid_policykit(self):
        return self.call_dbus_iface("kydroid_policykit")

    def call_auth_with_policykit(self,text,action_id):
        return True
        # return  self.call_dbus_iface("auth_with_policykit",(text,action_id))
        list = [text,action_id]
        signature_data = self.add_key(list)
        try:
            return self.iface.auth_with_policykit(text,action_id,signature_data)
        except:
            self.init_dbus_ifaces()
            try:
                return self.iface.auth_with_policykit(text,action_id,signature_data)
            except:
                return False
        # return True

    def call_copy_file_to_archives(self,app_fullname,src_path,install_appname,need_emit_signal):
        try:
            return self.iface.copy_file_to_install(app_fullname,src_path,install_appname,need_emit_signal)
        except:
            self.init_dbus_ifaces()
            try:
                self.iface.copy_file_to_install(app_fullname, src_path, install_appname, need_emit_signal)
            except:
                pass
            return False

    def call_copy_file_to_apk_tmp(self,app_fullname,src_path,dest_path,install_appname):
        # def copy_apk_to_install(self, app_fullname, src_path,dest_path,install_appname):
        cmd = "mv " + src_path + "/" + app_fullname +" "+ dest_path
        os.system(cmd)
        # self.copy_finish(app_fullname,install_appname)
        self.call_copy_finish(app_fullname,install_appname)
        # return self.iface.copy_apk_to_install(app_fullname, src_path,dest_path,install_appname)
        return

    def call_copy_finish(self,app_fullname,install_appname):
        self.copy_finish.emit(app_fullname,install_appname)

   #
    # 函数：软件状态的信号响应
    #
    def _on_software_fetch_signal(self, type, kwarg):
        sendType = "fetch"
        appname = str(kwarg['download_appname'])
        percent = float(str(kwarg['download_percent']))
        action = str(kwarg['action'])
        sendMsg = AptProcessMsg[str(type)]
        if( type == "down_pulse"):
            sizepercent = str(int(kwarg['download_bytes'])/1024) + "/" + str(int(kwarg['total_bytes'])/1024)
            itempercent = str(int(kwarg['download_items'])) + "/" + str(int(kwarg['total_items']))
            sendMsg = sendMsg + "(" + sizepercent + ")" + "(" + itempercent + ")"
        # if type == "down_fetch":
        #     if (Globals.DEBUG_SWITCH):
        #         LogManager(INFO_LOG,level='info').logger.info(_("downloading"+str(kwarg["uri"])))

        self.dbus_apt_process.emit(appname,sendType,action,percent,sendMsg)

    #
    # 函数：apt调用的返回信号响应
    #
    def _on_software_apt_signal(self,type, kwarg):
        sendType = "apt"
        appname = str(kwarg['apt_appname'])
        sendMsg  = ""
        percent = float(str(kwarg['apt_percent']))
        action = str(kwarg['action'])
        sendMsg = AptActionMsg[action] + AptProcessMsg[str(type)]

        self.dbus_apt_process.emit(appname,sendType,action,percent,sendMsg)


    #
    # 函数：auth信号响应
    #
    def _on_software_auth_signal(self,type, kwarg):

        sendType = "auth"
        appname = str(kwarg['appname'])
        #sendMsg  = "操作取消"
        sendMsg = _("Operation canceled")
        action = str(kwarg['action'])
        if type == "auth_cancel":
            sendType = "cancel"

        self.dbus_apt_process.emit(appname,sendType,action,0,sendMsg)

    #
    # 函数：调用kydroid的dbus接口
    #
    def call_kydroid_dbus_iface(self, funcname, kwargs=None,kwargs2=None,kwargs3=None):
        if self.kydroid_iface is None:
            return None
        func = getattr(self.kydroid_iface,funcname)
        if func is None:
            return None

        res = None
        try:
            res = func(kwargs,kwargs2,kwargs3)
        except dbus.DBusException as e:
            Globals.LOG_ERROR.logger.error("apt-daemon kydroid dbus exception:%s" % str(e))
            return None
        return res

    #
    # 函数：打开软件商店用户手册
    #
    @staticmethod
    def show_user_guide(msg):

        dbus_service = "com.kylinUserGuide.hotel" + "_" + str(os.getuid())
        dbus_path = "/"
        dbus_interface = "com.guide.hotel"

        try:
            message = QtDBus.QDBusMessage.createMethodCall(dbus_service, dbus_path, dbus_interface, "showGuide")
            message << msg
            QtDBus.QDBusConnection.sessionBus().call(message)

        except Exception as e:
            Globals.LOG_ERROR.logger.error("dbus exception:%s" % str(e))
            return False

        return True


if __name__ == "__main__":
    # dbus_service = Dbus_Service()

    Dbus_Service.show_user_guide()




#!/usr/bin/python3
# -*- coding: utf-8 -*-

from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from PyQt5 import QtCore, QtGui
from ui.starwidget import StarWidget
from ui.messagebox import MessageBox
from models.enums import Signals
# from ui.loadingdiv import *
import sip
import math
import gettext
gettext.bindtextdomain("ubuntu-kylin-software-center", "/usr/share/locale")
gettext.textdomain("ubuntu-kylin-software-center")
_ = gettext.gettext
from models.application import *
from models.enums import UBUNTUKYLIN_DATA_PATH
# from models.enums import CACHE_DIR,BUTTON_PRESSED,BUTTON_ENTER,BUTTON_LEAVE,BUTTON_DISABLE,AppMsg
#


class Ahead_Review_Widget(QWidget,Signals):
    ad_move_left_time = QTimer()
    ad_move_right_time = QTimer()
    final_x_position = 0
    x = 0
    maxpage = ''
    mainwindow = ''
    debfile = ''
    sshotcount = 0
    bigsshot = ''
    reviewpage = ''
    currentreviewready = ''
    debfile = None
    Row_er = 0
    Expand_height = 0
    is_clean_star = 0
    add_revieheight = 0
    Line_height = 0
    text_lidit = ""
    dest_str = ""
    SET_ISNTALL = False
    count = 0
    commend_height = 260
    # i = 0
    reviewlist1=[]
    alist = []
    def __init__(self,reviewlist1,app_detail,parent=None):
        QWidget.__init__(self,parent)
        # self.dblist = []
        # applist1 = []
        # self.app = app
        # self.reviewpage = reviewpage
        self.app_detail = app_detail
        self.ratings_average = 0
        self.messageBox = MessageBox(self)
        self.mainwindow = parent
        self.mainwindow.installEventFilter(self)
        self.mouse_press = False
        # self.setFixedSize(1340,278)

        self.setWindowFlags(Qt.FramelessWindowHint)
        # self.setAttribute(Qt.WA_TranslucentBackground)
        #
        self.left_btn = QPushButton(self.mainwindow)
        self.left_btn.setFocusPolicy(Qt.NoFocus)
        self.left_btn.resize(31, 31)
        if(Globals.THEME_COLOR == 0):
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_false_btn.png);}")
        else:
            self.left_btn.setStyleSheet(
                "QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_false_dark.png);}")
        self.left_btn.move(10, int((self.commend_height - self.left_btn.height()) / 2))
        self.left_btn.hide()
        self.left_btn.setEnabled(False)



        self.widget = QWidget(self.mainwindow)
        self.widget.resize(self.app_detail.editorial_comments_widget.width(), self.commend_height)
        self.widget.move(self.left_btn.width()+20,0)
        self.widget.setStyleSheet(".QWidget{border-radius:12px;background-color:transparent;}")
        self.widget.installEventFilter(self)
        self.review_card_width = int((self.widget.width() - 20) / 2)

        self.right_btn = QPushButton(self.mainwindow)
        self.right_btn.setFocusPolicy(Qt.NoFocus)
        self.right_btn.resize(31, 31)
        if (Globals.THEME_COLOR == 0):
            self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_true_btn.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/right_true_clicked_btn.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/right_true_clicked_btn.png);}")
        else:
            self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_true_btn_dark.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/right_true_click_dark.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/right_true_click_dark.png);}")
        self.right_btn.move(self.widget.width() + self.widget.x() + 10,
                            int((self.commend_height - self.right_btn.height()) / 2))
        self.right_btn.hide()

        # 评论的大widget，对applist其中每一条评论都实例化一个小widget

        self.review_widget1 = QWidget(self.widget)
        self.review_widget1.setStyleSheet(".QWidget{border:1px ;border-radius:12px;background:transparent;}")
        self.review_widget1.move(0, 0)
        self.review_widget1.installEventFilter(self)

        self.right_btn.clicked.connect(self.slot_click_right_btn)
        self.left_btn.clicked.connect(self.slot_click_left_btn)
        self.ad_move_left_time.timeout.connect(self.slot_slide_left)
        self.ad_move_right_time.timeout.connect(self.slot_slide_right)


        self.total_review_count = len(reviewlist1)
        self.show_review_count = 0
        self.right_click_count = 0

        self.show_review_x = 0
        self.show_review_y = 0
        self.reviewlist1 = reviewlist1

        # for i in range(self.total_review_count):
        #     self.applist1.append(i)
        #     self.innit_ui(self.applist1)

        self.innit_ui(self.reviewlist1)


    def innit_ui(self,reviewlist):
        self.review_widget1.resize(self.review_card_width * (len(reviewlist)) + 20 * (len(reviewlist) -1),self.commend_height)
        for k in range(self.total_review_count):
            ratings_average = self.reviewlist1[k].user_rating
            self.review1 = ReviewLabel(ratings_average, self.review_card_width, reviewlist[k], self.app_detail,self.review_widget1)
            self.review1.move(self.review1.width() * k + 20 * k, 0)

        self.show_review_count = 10
        self.show_review_x = self.review1.width() * 10 + 20 * 10

        self.count = len(reviewlist)

    def re_move(self):
        self.left_btn.move(10, int((self.commend_height - self.left_btn.height()) / 2))
        self.widget.resize(self.app_detail.editorial_comments_widget.width(), self.commend_height)
        self.widget.move(self.left_btn.width() + 20, 0)
        self.review_card_width = int((self.widget.width() - 20) / 2)
        self.right_btn.move(self.widget.width() + self.widget.x() + 10,
                            int((self.commend_height - self.right_btn.height()) / 2))
        self.review_widget1.move(0, 0)
        self.review_widget1.setFixedSize(self.review_card_width * self.count + 20 * (self.count - 1), self.commend_height)
        reviews = self.review_widget1.children()
        n = 0
        for i in reviews:
            i.setFixedSize(self.review_card_width,self.commend_height)
            i.ReviewWidget.resize(self.review_card_width,self.commend_height)
            i.re_move()
            i.move(self.review_card_width * n + 20 * n, 0)
            n = n+1
        if(self.count <= 2 ):
            self.right_btn.setEnabled(False)
            self.left_btn.setEnabled(False)
            self.check_btn_styleSheet()
        else:
            self.right_btn.setEnabled(True)
            self.left_btn.setEnabled(False)
            self.check_btn_styleSheet()

    def check_btn_styleSheet(self):
        if(Globals.THEME_COLOR == 0):
            if(self.left_btn.isEnabled()):
                self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_true_btn.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/left_true_clicked_btn.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/left_true_clicked_btn.png);}")
            else:
                self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_false_btn.png);}")
            if(self.right_btn.isEnabled()):
                self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_true_btn.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/right_true_clicked_btn.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/right_true_clicked_btn.png);}")
            else:
                self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_false_btn.png);}")
        else:
            if (self.left_btn.isEnabled()):
                self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_true_btn_dark.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/left_true_click_dark.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/left_true_click_dark.png);}")
            else:
                self.left_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/left_false_dark.png);}")
            if (self.right_btn.isEnabled()):
                self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_true_btn_dark.png);}QPushButton:hover{background-color:transparent;border-radius:20px;image: url(./res/right_true_click_dark.png);}QPushButton:pressed{background-color:transparent;border-radius:20px;image: url(./res/right_true_click_dark.png);}")
            else:
                self.right_btn.setStyleSheet("QPushButton{background-color:transparent;border-radius:20px;image: url(./res/right_false_dark.png);}")


    def get_review(self):
        # 没有更多评论啦~
        if self.show_review_count >= self.total_review_count:
            return
        # 更多评论不够10条
        elif self.show_review_count + 10 >= self.total_review_count:
            num = self.total_review_count - self.show_review_count
            for k in range(num):
                self.review1 = ReviewLabel(self.ratings_average,self.review_card_width, self.reviewlist1[self.show_review_count + k],self.app_detail, self.review_widget1)
                self.review1.move(self.show_review_x + self.review1.width() * k + 20 * k, 0)
                self.review1.raise_()
                self.review1.show()

            self.show_review_count += num
            self.show_review_x += self.review1.width() * num + 20 * num
        # 还能加载10条评论
        else:
            num = 10
            for k in range(num):
                self.review1 = ReviewLabel(self.ratings_average,self.review_card_width, self.reviewlist1[self.show_review_count + k], self.app_detail,self.review_widget1)
                self.review1.move(self.show_review_x + self.review1.width() * k + 20 * k, 0)
                self.review1.raise_()
                self.review1.show()

            self.show_review_count += num
            self.show_review_x += self.review1.width() * num + 20 * num

    

    # 函数：点击右边按钮
    def slot_click_right_btn(self):
        # self.right_btn.setEnabled(False)
        self.right_click_count += 1
        if(len(self.reviewlist1) >2):
            if ((self.right_click_count + 1) % 5) == 0:
                self.right_btn.clicked.connect(self.get_review)

            self.left_btn.setEnabled(True)
            self.check_btn_styleSheet()
            x = self.review_widget1.x()
            if (self.final_x_position == 0):
                self.final_x_position =-(self.review_card_width*2+2*20)
                Globals.POS = self.final_x_position
                self.ad_move_left_time.start(1)
            else:
                self.final_x_position = self.review_widget1.x() - (self.review_card_width*2+20*2)
                Globals.POS = self.final_x_position
                self.ad_move_left_time.start(1)

    #
    # #
    # 函数：向左滑动:1.左边滑动。2.滑到终点按钮置灰。3.滑到一般预加载
    def slot_slide_left(self):
        self.ad_move_left_time.stop()
        self.final_x_position = Globals.POS
        self.right_btn.setEnabled(False)
        self.left_btn.setEnabled(False)

        x = self.review_widget1.x()
        if(x-67 > self.final_x_position):
            self.review_widget1.move(x - 67, 0)
        else:
            self.review_widget1.move(self.final_x_position, 0)
        self.ad_move_left_time.start(1)
        x = self.review_widget1.x()
        if (x == self.final_x_position) :
            self.ad_move_left_time.stop()
            self.right_btn.setEnabled(True)
            self.left_btn.setEnabled(True)
            self.check_btn_styleSheet()


            # self.final_x_position = 0
        if self.count % 2 == 0:
            if (x == -(self.review_widget1.width() - (self.review_card_width*2+20))):
                self.right_btn.setEnabled(False)
                self.check_btn_styleSheet()

        else:
            if (x == -(self.review_widget1.width() - self.review_card_width)):
                self.right_btn.setEnabled(False)
                self.check_btn_styleSheet()

        # if x == - self.review_widget1.width()/2 :
        #     self.ma
        #     self.m_parent.mw.at.appmgr.get_application_reviews(self.app_msg.pkgname, page=self.reviewpage)
        #     self.get_review()


    #
    # 函数：点击向左的箭头
    #
    def slot_click_left_btn(self):
        self.right_btn.setEnabled(True)
        self.check_btn_styleSheet()

        x = self.review_widget1.x()

        if (self.final_x_position == 0):
            self.final_x_position = (self.review_card_width*2+20*2)
            Globals.POS = self.final_x_position
            self.ad_move_right_time.start(0.1)
        else:
            self.final_x_position = self.review_widget1.x() + (self.review_card_width*2+20*2)
            Globals.POS = self.final_x_position
            self.ad_move_right_time.start(0.1)

    #
    # 函数向右滑动

    def slot_slide_right(self):
        self.ad_move_right_time.stop()
        self.final_x_position = Globals.POS
        self.left_btn.setEnabled(False)
        self.right_btn.setEnabled(False)

        x = self.review_widget1.x()
        if(x+67 < self.final_x_position):
            self.review_widget1.move(x + 67, 0)
        else:
            self.review_widget1.move(self.final_x_position, 0)
        # self.review_widget1.move(x + 67, 0)
        self.ad_move_right_time.start(1)
        x = self.review_widget1.x()
        if (x ==self.final_x_position):
            self.ad_move_right_time.stop()
            self.left_btn.setEnabled(True)

            self.right_btn.setEnabled(True)
            self.check_btn_styleSheet()

            # self.final_x_position = 0
        if (x == 0):
            self.left_btn.setEnabled(False)
            self.check_btn_styleSheet()


    #
    # 函数：鼠标按下事件
    #
    def mousePressEvent(self, event):
        if event.button() == Qt.LeftButton:
            pass
        else:
            event.ignore()

    def enterEvent(self, event):
        if (len(self.reviewlist1) > 2):
            self.left_btn.show()
            self.right_btn.show()

    def eventFilter(self, obj, event):
        if (event.type() == event.Enter):
            if (obj == self.review_widget1 or obj == self.widget or obj == self.mainwindow):
                if (len(self.reviewlist1) > 2):
                    self.left_btn.show()
                    self.right_btn.show()
        if (event.type() == event.Leave):
            if (obj == self.mainwindow):
                self.left_btn.hide()
                self.right_btn.hide()
        return False


class ReviewLabel(QWidget):

    commend_height=260
    agree_with = False
    disagree = False

    def __init__(self, ratings_average, width,reviewlist, app_detail ,parent=None):
        QWidget.__init__(self,parent)
        # strstr = ""
        # k = 0
        self.m_parent = parent
        self.app_detail = app_detail
        self.ratings_average = ratings_average
        self.comment_line_num = 6
        self.w = width
        self.reviewlist = reviewlist
        self.init_ui()

    def init_ui(self):
        self.reviewlist = self.reviewlist
        self.ReviewWidget = QWidget(self)
        self.ReviewWidget.resize(self.w,self.commend_height)
        self.ReviewWidget.move(0,0)
        # self.ReviewWidget.setStyleSheet("QWidget{border:none;border-radius:12px;background-color:#F4F4F4}")
        Globals.ADD_WIDGET.append(self.ReviewWidget)
        self.userHead = QLabel(self.ReviewWidget)
        self.userHead.setGeometry(QtCore.QRect(16, 16, 40, 40))
        self.userHead.setAttribute(Qt.WA_TranslucentBackground)
        self.userName = QLabel(self.ReviewWidget)
        self.userName.setGeometry(QtCore.QRect(66, 16, 166, 22))
        self.userName.setAttribute(Qt.WA_TranslucentBackground)
        self.me_label = QLabel(self.ReviewWidget)
        self.me_label.resize(20,18)
        # self.me_label.setAttribute(Qt.WA_TranslucentBackground)
        self.me_label.move(self.userName.x()+self.userName.width()+8,self.userName.y()+int((self.userName.height()-self.me_label.height())/2))
        if(Globals.LAN == "zh_CN"):
            self.me_label.setStyleSheet("QLabel{background-image:url('res/myself_zh.png')}")
        else:
            self.me_label.setStyleSheet("QLabel{background-image:url('res/myself_en.png')}")
        self.comment = QLabel(self.ReviewWidget)
        self.comment.move(16,66)
        self.comment.setAttribute(Qt.WA_TranslucentBackground)
        # self.comment.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.comment.setContextMenuPolicy(Qt.NoContextMenu)
        # setTextInteractionFlags(Qt::TextSelectableByMouse)
        # self.comment.setFixedSize(484,126)
        # self.comment.show()
        self.createDate = QLabel(self.ReviewWidget)
        self.createDate.setAttribute(Qt.WA_TranslucentBackground)
        # self.createDate.setGeometry(QtCore.QRect(345, 16, 155, 20))
        # self.createDate.move(516-16-self.createDate.width(),16)
        # self.createDate.setFixedHeight(20)

        self.agree_with_btn = QPushButton(self.ReviewWidget)
        self.agree_with_btn.setFocusPolicy(Qt.NoFocus)
        # self.agree_with_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.agree_with_btn.resize(20, 20)
        self.agree_with_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_agree.png');}QPushButton:hover{background-image: url(./res/hover_agree.png);}QPushButton:pressed{background-image: url(./res/press_agree.png);}")
        self.agree_with_btn.move(16, self.ReviewWidget.height() - self.agree_with_btn.height() - 16)
        self.agree_with_btn.clicked.connect(lambda :self.app_detail.slot_click_agree_with_btn(self.reviewlist,self))

        self.agree_with_count_label = QLabel(self.ReviewWidget)
        self.agree_with_count_label.setAttribute(Qt.WA_TranslucentBackground)
        self.agree_with_count_label.resize(20, 20)
        self.agree_with_count_label.setStyleSheet(
            "QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;}")
        self.agree_with_count_label.move(self.agree_with_btn.x() + self.agree_with_btn.width() + 4,
                                         self.agree_with_btn.y() + int(
                                             (self.agree_with_btn.height() - self.agree_with_count_label.height()) / 2))

        self.disagree_btn = QPushButton(self.ReviewWidget)
        self.disagree_btn.setFocusPolicy(Qt.NoFocus)
        # self.agree_with_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.disagree_btn.resize(20, 20)
        self.disagree_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_disagree.png');}QPushButton:hover{background-image: url(./res/hover_disagree.png);}QPushButton:pressed{background-image: url(./res/press_disagree.png);}")
        self.disagree_btn.move(79, self.ReviewWidget.height() - self.agree_with_btn.height() - 16)
        self.disagree_btn.clicked.connect(lambda :self.app_detail.slot_click_disagree_btn(self.reviewlist,self))

        self.disagree_count_label = QLabel(self.ReviewWidget)
        self.disagree_count_label.setAttribute(Qt.WA_TranslucentBackground)
        self.disagree_count_label.resize(20, 20)
        self.disagree_count_label.setStyleSheet(
            "QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;}")
        self.disagree_count_label.move(self.disagree_btn.x() + self.disagree_btn.width() + 4,
                                       self.disagree_btn.y() + int(
                                           (self.disagree_btn.height() - self.disagree_count_label.height()) / 2))

        self.delete_btn = QPushButton(self.ReviewWidget)
        self.delete_btn.setFocusPolicy(Qt.NoFocus)
        self.delete_btn.setAttribute(Qt.WA_TranslucentBackground)
        self.delete_btn.setStyleSheet("QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;}")
        self.delete_btn.setText(_("Delete"))
        self.delete_btn.adjustSize()
        self.delete_btn.move(self.ReviewWidget.width()-16-self.delete_btn.width(),self.ReviewWidget.height()-self.delete_btn.height()-16)
        self.delete_btn.clicked.connect(lambda :self.app_detail.slot_delete_comment(self.reviewlist,self.app_detail.app_msg.pkgname))


        self.star = StarWidget('small', self.ratings_average, self.ReviewWidget)
        self.star.move(67, 43)
        self.star.setFixedSize(98,14)
        self.star.setStyleSheet("QWidget{background-color:transparent;}")
        self.setFixedSize(self.w,self.commend_height)
        self.star.hide()

        # self.comment.setAlignment(Qt.AlignVCenter)
        # self.comment.setWordWrap(True)
        self.comment.setAlignment(Qt.AlignLeft | Qt.AlignTop)
        self.comment.setWordWrap(True)
        # self.comment.setAlignment(Qt.AlignTop)
        self.comment.setTextInteractionFlags(Qt.TextSelectableByMouse)
        self.userName.setStyleSheet("QLabel{font-size: 16px;font-weight: 500;color: #303133;line-height: 22px;}")
        Globals.ADD_WIDGET_USERNAME.append(self.userName)
        self.comment.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 21px;}")
        Globals.ADD_WIDGET_COMMENT.append(self.comment)
        if (self.reviewlist.user_profile != ''):
            if (self.reviewlist.user_display == Globals.LOGIN_ACCOUNT):
                user_profile = UBUNTUKYLIN_DATA_PATH + "user_profile/" + Globals.USER_PROFILE
            else:
                user_profile = UBUNTUKYLIN_DATA_PATH + "user_profile/" + self.reviewlist.user_profile
            pix2 = QPixmap(user_profile)
            fitpixmap_userIcon = pix2.scaled(40, 40, Qt.IgnoreAspectRatio, Qt.SmoothTransformation)
            fitpixmap_userIcon = self.PixmapToRound(fitpixmap_userIcon, 40,
                                                    40)
            self.userHead.setPixmap(fitpixmap_userIcon)
        else:
            self.userHead.setStyleSheet("QLabel{background-image:url('res/userhead.png')}")
        self.createDate.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;}")
        # Globals.ADD_WIDGET_DATE.append(self.createDate)


        self.userName.setText(self.reviewlist.user_display)
        self.userName.adjustSize()
        self.me_label.move(self.userName.x() + self.userName.width() + 8,
                           self.userName.y() + int((self.userName.height() - self.me_label.height()) / 2))

        if (str(self.reviewlist.account) == str(Globals.ACCOUNT)):
            self.me_label.show()
            self.delete_btn.show()
        else:
            self.me_label.hide()
            self.delete_btn.hide()
        self.createDate.setText(self.reviewlist.date)
        # print("评论的长度",len(self.createDate.text()))
        self.createDate.move(self.w-16-len(self.createDate.text()*7),16)
        self.createDate.setFixedHeight(20)

        if (self.reviewlist.like_count != None):
            self.comment.setGeometry(QtCore.QRect(16, 66, int(self.ReviewWidget.width()-16*2), int(self.delete_btn.y()-self.comment.y())))
            self.agree_with_count_label.setText(str(self.reviewlist.like_count))
            self.agree_with_count_label.adjustSize()
            self.disagree_count_label.setText(str(self.reviewlist.unlike_count))
            self.disagree_count_label.adjustSize()
            self.comment_line_num = 6
        else:
            self.comment.setGeometry(QtCore.QRect(16, 66, self.ReviewWidget.width() - 16 * 2, 150))
            self.agree_with_count_label.hide()
            self.agree_with_btn.hide()
            self.disagree_count_label.hide()
            self.disagree_btn.hide()
            self.comment_line_num = 7
        if (self.reviewlist.property != None and str(self.reviewlist.property) == "1"):
            self.agree_with = True
            self.click_agree_over()
        elif (self.reviewlist.property != None and str(self.reviewlist.property) == "2"):
            self.disagree = True
            self.click_disagree_over()

        self.comment.setText(self.reviewlist.content)
        self.setLongTextToElideFormat(self.comment, self.reviewlist.content,self.comment_line_num)
        self.show_dark_mode()

    def check_account(self):
        if (str(self.reviewlist.account) == str(Globals.ACCOUNT)):
            self.me_label.show()
            self.delete_btn.show()
        else:
            self.me_label.hide()
            self.delete_btn.hide()

    def click_disagree_over(self):
        self.agree_with_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_agree.png');}QPushButton:hover{background-image: url(./res/hover_agree.png);}QPushButton:pressed{background-image: url(./res/press_agree.png);}")
        self.disagree_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image: url(./res/press_disagree.png);}")

    def click_agree_over(self):
        self.agree_with_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image: url(./res/press_agree.png);}")
        self.disagree_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_disagree.png');}QPushButton:hover{background-image: url(./res/hover_disagree.png);}QPushButton:pressed{background-image: url(./res/press_disagree.png);}")

    def init_btn_statue(self):
        self.agree_with_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_agree.png');}QPushButton:hover{background-image: url(./res/hover_agree.png);}QPushButton:pressed{background-image: url(./res/press_agree.png);}")
        self.disagree_btn.setStyleSheet(
            "QPushButton{border: 0px;font-size: 14px;font-weight: 500;color: #3790FA;line-height: 20px;background-image:url('res/default_disagree.png');}QPushButton:hover{background-image: url(./res/hover_disagree.png);}QPushButton:pressed{background-image: url(./res/press_disagree.png);}")


    def re_move(self):
        self.comment.setGeometry(QtCore.QRect(16, 66, self.ReviewWidget.width() - 16 * 2, self.comment.height()))
        print("small",self.comment.height())
        self.createDate.move(self.width() - 16 - len(self.createDate.text() * 7), 16)
        self.createDate.setFixedHeight(20)
        self.setLongTextToElideFormat(self.comment, self.reviewlist.content,self.comment_line_num)
        self.delete_btn.move(self.ReviewWidget.width() - 16 - self.delete_btn.width(),
                             self.ReviewWidget.height() - self.delete_btn.height() - 16)

    def setLongTextToElideFormat(self, label, te,line_num=7):

        te = te.replace("\n", ",").replace("\r", "").replace("\t", "").replace(";","；").replace(",","，")
        metrics = QFontMetrics(label.font())
        elidedText = metrics.elidedText(te, Qt.ElideRight, label.width()*line_num)


        label.setText(elidedText)
        if ("…" in elidedText):

            count = 0
            set_text = ""
            for i in te:
                if(count < 35):
                    set_text = set_text + i
                elif(count == 35):
                    set_text = set_text + i + "\n"
                    count = 0
                count = count +1
            QToolTip.setFont(QFont("SansSerif", 10))
            self.setToolTip(set_text)
        else:
            self.splitText(label, te, label.width())


    #label:控件，text：文字，nlabelsize：控件宽
    def splitText(self,label,text,nlabelsize):
        font = QFont()
        font.setPixelSize(14)
        m_font = QFontMetrics(font)
        nTextSize = m_font.width(text)
        text = label.text()
        label.setText("")
        split_num = 1
        if (label.width() % 2 != 0):
            width = label.width() - 1
        else:
            width = label.width()
        for i in text:
            pre_text = label.text()
            label.setText(label.text() + i)
            s = m_font.boundingRect(label.text()).size().width()
            if (s <= width * split_num+split_num-1):
                pass
            else:
                label.setText(pre_text + "\n" + i)
                split_num = split_num + 1



    def show_dark_mode(self):
        if (Globals.THEME_COLOR == 0):
            self.setStyleSheet(".QWidget{border:none;border-radius:12px;background-color:#F4F4F4}")
            self.ReviewWidget.setStyleSheet(".QWidget{border:none;border-radius:12px;background-color:#F4F4F4}")
            self.userName.setStyleSheet("QLabel{font-size: 16px;font-weight: 500;color: #303133;line-height: 22px;}")
            self.comment.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #606265;line-height: 21px;}")
        else:
            self.setStyleSheet(".QWidget{border:none;border-radius:12px;background-color:#393939}")
            self.ReviewWidget.setStyleSheet(".QWidget{border:none;border-radius:12px;background-color:#393939}")
            self.userName.setStyleSheet(
                "QLabel{font-size: 16px;font-weight: 500;color: #F9F9F9;line-height: 22px;}")
            self.comment.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 21px;}")

    # 将图片切割成圆角
    def PixmapToRound(self, src, x, y):
        pixmapa = QPixmap(src)
        pixmap = QPixmap(x, y)
        pixmap.fill(Qt.transparent)
        painter = QPainter(pixmap)
        painter.setRenderHint(QPainter.Antialiasing, True)
        # painter.setRenderHints(QPainter.Antialiasing | QPainter.SmoothPixmapTransform)
        path = QPainterPath()
        rect = QRectF(0, 0, x, y)
        path.addRoundedRect(rect, 20.0, 20.0)
        painter.setClipPath(path)
        painter.drawPixmap(0, 0, x, y, pixmapa)

        return pixmap


#!/usr/bin/python3
# -*- coding: utf-8 -*
import dbus
import os
import sys
import shutil
import locale
import datetime
import time
from gi.repository import GObject
import logging

from PyQt5.QtCore import *
from PyQt5 import QtDBus
UBUNTUKYLIN_SOFTWARECENTER_ACTION = 'com.kylin.softwarecenter.action'


import multiprocessing

from dbus.mainloop.glib import DBusGMainLoop, threads_init
from models.globals import Globals
import gettext
gettext.textdomain("kylin-software-center")
_ = gettext.gettext
UBUNTUKYLIN_SERVICE_PATH = "com.kylin.softwarecenter.runapp"
UBUNTUKYLIN_INTERFACE_PATH = "com.kylin.softwarecenter.runapp"


class Run_Dbus_Service(QObject):

    def __init__(self):
        QObject.__init__(self)
        locale.setlocale(locale.LC_ALL, "zh_CN.UTF-8")

        self.iface = None
        self.init_dbus_ifaces()

    #
    # 函数：初始化dbus接口
    #
    def init_dbus_ifaces(self):
        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_run_dbus_ifaces (" + datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f") + ")")
            DBusGMainLoop(set_as_default=True)
            threads_init()
            bus = dbus.SessionBus()
        except Exception as e:
            Globals.LOG_ERROR.logger.error("dbus exception:%s" % str(e))
            return False

        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces call blocking (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            bus.call_blocking(UBUNTUKYLIN_SERVICE_PATH, '/com/kylin/softwarecenter/runapp', UBUNTUKYLIN_INTERFACE_PATH, 'wakeup', None, (), timeout=5)
        except dbus.DBusException as e:
            Globals.LOG_ERROR.logger.error("InstallBackend DBusConnection call blocking exception: %s" % str(e))
            return False

        try:
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces bus.get_object (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            obj = bus.get_object(UBUNTUKYLIN_SERVICE_PATH, '/com/kylin/softwarecenter/runapp')
            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces dbus.Interface (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
            self.iface = dbus.Interface(obj, dbus_interface=UBUNTUKYLIN_INTERFACE_PATH)

            Globals.LOG_INFO.logger.info("InstallBackend init_dbus_ifaces finished (%s)" % datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S.%f"))
        except dbus.DBusException as e:
            Globals.LOG_ERROR.logger.error("InstallBackend DBusConnection exception: %s" % str(e))
            return False

        return True

    def call_exit(self):
        self.iface.stop()

    def call_run_app(self,cmd):
        self.iface.run_app(cmd)

    def call_run_ksc_app(self,cmd):
        self.iface.run_ksc_app(cmd)


#!/usr/bin/python3
# -*- coding: utf-8 -*-


from PyQt5.QtGui import *
from PyQt5.QtCore import *
from PyQt5.QtWidgets import *
from models.globals import *

import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

class Historical_Insatallation_widget(QWidget):
    click_history_widget_btn = pyqtSignal(str)
    def __init__(self, w, h, parent=None):
        QWidget.__init__(self, parent)
        self.w = w
        self.h = h
        self.resize(self.w,self.h)
        self.m_parent = parent
        self.now_btn_name = "local"

        self.label_local = QPushButton(self)
        self.label_local.setFocusPolicy(Qt.NoFocus)
        self.label_local.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
        #self.label_local.setText(_("本机安装历史"))
        self.label_local.setText(_("Installation history of the machine"))
        self.label_local.adjustSize()
        self.label_local.move(20,0)
        self.label_local.show()
        self.label_local.clicked.connect(self.slot_click_local_btn)

        self.label_other = QPushButton(self)
        self.label_other.setFocusPolicy(Qt.NoFocus)
        self.label_other.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
        #self.label_other.setText(_("其他设备安装历史"))
        self.label_other.setText(_("Cloud installation history"))
        self.label_other.adjustSize()
        self.label_other.move(self.label_local.x()+self.label_local.width()+20,self.label_local.y())
        self.label_other.show()
        self.label_other.clicked.connect(self.slot_click_other_btn)
        if(Globals.NEED_SHOW_KYLINID == False):
            self.label_other.hide()

        self.app_card_widget = QWidget(self)
        self.app_card_widget.resize(self.width(), self.height()-self.label_other.height()-20)
        self.app_card_widget.move(int((self.width() - self.app_card_widget.width()) / 2), self.label_other.y()+self.label_other.height())
        self.app_card_widget.setStyleSheet("QWidget{border-radius: 0px;background-color:transparent;}")
        self.app_card_widget.hide()

        self.text_prompt_label = QLabel(self)
        self.text_prompt_label.setAttribute(Qt.WA_TranslucentBackground)
        # self.text_prompt_label.setStyleSheet("QLabel{font-size: 14px;font-weight: 400;color: #BBBDCA;line-height: 24px;background-color:transparent;}")
        self.text_prompt_label.hide()

        self.no_app_tip_label = QLabel(self)
        self.no_app_tip_label.resize(240, 240)
        if(Globals.THEME_COLOR == 0):
            self.no_app_tip_label.setStyleSheet("QLabel{border-radius: 12px;background-image:url(./res/no_uninstall.png);background-color:transparent;}")
        else:
            self.no_app_tip_label.setStyleSheet("QLabel{border-radius: 12px;background-image:url(./res/no_uninstall_dark.png);background-color:transparent;}")
        self.no_app_tip_label.hide()

        self.set_dark_mode()

    def set_dark_mode(self):
        if(Globals.THEME_COLOR == 0):
            self.label_local.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
            self.label_other.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
            self.text_prompt_label.setStyleSheet( "QLabel{font-size: 14px;font-weight: 400;color: #BBBDCA;line-height: 24px;background-color:transparent;}")
        else:
            self.label_local.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
            self.label_other.setStyleSheet(
                "QPushButton{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
            self.text_prompt_label.setStyleSheet(
                "QLabel{font-size: 14px;font-weight: 400;color: #C0C4CC;line-height: 24px;background-color:transparent;}")

    def re_move(self):
        self.label_local.move(20, 20)
        self.label_other.move(self.label_local.x() + self.label_local.width() + 20, self.label_local.y())
        self.app_card_widget.resize(self.width(), self.height() - self.label_other.height() - 20)
        self.app_card_widget.move(int((self.width() - self.app_card_widget.width()) / 2),
                                  self.label_other.y() + self.label_other.height())
        self.no_app_tip_label.move(int(self.app_card_widget.x() + int((self.app_card_widget.width() - self.no_app_tip_label.width()) / 2)),int(((self.height() - self.no_app_tip_label.height()) / 2)-((self.m_parent.btn_widget.y()+self.m_parent.btn_widget.height())/2)))
        self.text_prompt_label.move(int((self.width() - self.text_prompt_label.width()) / 2),self.no_app_tip_label.y() + self.no_app_tip_label.height() + 10)

    def reload_size(self):
        self.setFixedHeight(self.h)
        self.no_app_tip_label.move(self.app_card_widget.x() + int((self.app_card_widget.width() - self.no_app_tip_label.width()) / 2),int((self.height() - self.no_app_tip_label.height()) / 2)-((self.m_parent.btn_widget.y()+self.m_parent.btn_widget.height())/2))

    def slot_click_local_btn(self):
        self.now_btn_name = "local"
        self.label_local.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color:transparent;color: #3790FA;}")
        self.label_other.setStyleSheet("QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
        self.click_history_widget_btn.emit(_("Installation"))

    def slot_click_other_btn(self):
        self.now_btn_name = "other"
        self.label_other.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;line-height: 20px;text-align: center;background-color:transparent;color: #3790FA;}")
        self.label_local.setStyleSheet(
            "QPushButton{font-size: 14px;font-weight: 400;color: #8F9399;line-height: 20px;text-align: center;background-color:transparent;}QPushButton:hover{color: #3790FA;}QPushButton:pressed{color: #3790FA;}")
        self.click_history_widget_btn.emit(_("Installation"))



#!/usr/bin/python3
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:
#     Shine Huang<shenghuang@ubuntukylin.com>
# Maintainer:
#     Shine Huang<shenghuang@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys
import os
import stat
import shutil
import time
import configparser
from utils.log_manager import LogManager
from urllib import request
from PyQt5.QtCore import *
from models.enums import KYDROID_DOWNLOAD_PATH, Signals, AppActions, UBUNTUKYLIN_RES_ICON_PATH, UBUNTUKYLIN_CACHE_ICON_PATH, UBUNTUKYLIN_RES_PATH,UKSC_CACHE_DIR,ERR_LOG,INFO_LOG,DEBUG_LOG
import threading
from models.globals import Globals
from utils.png_to_svg import *
import gettext
gettext.bindtextdomain("kylin-software-center", "/usr/share/locale")
gettext.textdomain("kylin-software-center")
_ = gettext.gettext

# Globals.LOG_INFO = LogManager(INFO_LOG, level='info')
# Globals.LOG_ERROR = LogManager(ERR_LOG, level='error')

#
# 函数：下载源列表文件
#
def download_sourcelist():
    slurl = os.path.join(Globals.KYDROID_SOURCE_SERVER, 'kydroid-sourcelist')
    sllocal = os.path.join(UKSC_CACHE_DIR, 'kydroid-sourcelist')

    if not os.path.exists(sllocal):
        request.urlretrieve(slurl, sllocal)
    else:
        os.remove(sllocal)
        request.urlretrieve(slurl, sllocal)

class DownloadManager(QObject):
    apkpath = ""
    appname = ""
    size = 0
    appmgr = None

    def __init__(self, appmgr, apkInfo):
        # threading.Thread.__init__(self)
        QObject.__init__(self)
        self.apkpath = apkInfo.file_path
        self.appmgr = appmgr
        self.appname = apkInfo.pkgname
        self.size = apkInfo.installedSize
        self.apkInfo = apkInfo
        self.percent_num = 5

    def run(self):
        self.download_apk(self.apkpath)

    #
    # 函数：下载进度
    #
    def download_schedule(self, a, b, c):
        percent = a * b / self.size * 100
        if Globals.STOP_DOWNLOAD==True:
            sys.exit(0)
        if int(percent) == self.percent_num :
            self.percent_num += 5
            self.appmgr.apk_process.emit(self.appname, 'fetch', AppActions.INSTALL, int(percent), 'downloading apk file')

    #
    # 函数：下载安卓兼容应用
    #
    def download_apk(self, apkpath):
        if(Globals.DEBUG_SWITCH):
            #print("down apk start")
            Globals.LOG_INFO.logger.info(_("down apk start"))
        apkurl = os.path.join(Globals.KYDROID_SOURCE_SERVER, apkpath)
        filename = apkpath.split('/')[-1]
        apklocal = os.path.join(KYDROID_DOWNLOAD_PATH, filename)
        try:
            if(os.path.exists("/usr/share/kydroid/kydroid.conf")):
                cf = configparser.ConfigParser()
                cf.read("/usr/share/kydroid/kydroid.conf")
                kydroid_version = cf.get("image", "tag")
                kydroid_version = kydroid_version.split("-")[1].split(".")[0]
                if(int(kydroid_version) > 210401):
                    rtn = self.appmgr.kydroid_service.install_app(filename, self.appname,self.appname)
                else:
                    rtn = self.appmgr.kydroid_service.install_app(filename, self.appname)
            elif(os.path.exists("/usr/share/kmre/kmre.conf")):
                cf = configparser.ConfigParser()
                cf.read("/usr/share/kmre/kmre.conf")
                kydroid_version = cf.get("image", "tag")
                kydroid_version = kydroid_version.split("-")[1].split(".")[0]
                if (int(kydroid_version) > 210401):
                    rtn = self.appmgr.kydroid_service.install_app(filename, self.appname, self.appname)
                else:
                    rtn = self.appmgr.kydroid_service.install_app(filename, self.appname)
            else:
                rtn = self.appmgr.kydroid_service.install_app(filename, self.appname)
        except:
            rtn = self.appmgr.kydroid_service.install_app(filename, self.appname)
            pass
        # rtn = self.appmgr.kydroid_service.install_app(filename,self.appname)
        self.appmgr.apk_process.emit(self.appname, 'apt', AppActions.INSTALL, 75, 'install apk file')
        if (Globals.DEBUG_SWITCH):
            #print("APK install finished: ", rtn)
            Globals.LOG_INFO.logger.info(_("APK install finished: "+str(rtn)))
        if rtn == 1:
            # self.appmgr.normalcard_progress_finish.emit(self.appname)
            self.appmgr.apk_process.emit(self.appname, 'apt', AppActions.INSTALL, 200, 'install apk success')

            self.install_icon_desktop()
        else:
            if (Globals.DEBUG_SWITCH):
                #print("APK install failed.")
                Globals.LOG_INFO.logger.info(_("APK install failed."))
            self.appmgr.apk_process.emit(self.appname, 'apt', AppActions.INSTALL, -1, 'install apk failed')

    #
    # 函数：安装图标和desktop文件
    #
    # tmp目录下创建desktop文件，再转移到个人目录
    def install_icon_desktop(self):
        # 创建desktop文件
        user_uid = str(os.getuid())
        if(os.path.exists( "/tmp/" + user_uid) == False):
            os.mkdir( "/tmp/" + user_uid)
        self.tmp_desktop_file_path = "/tmp/" + user_uid + "/" + self.apkInfo.pkgname + ".desktop"
        appname = "kydroid_" + self.apkInfo.pkgname.split(".")[len(self.apkInfo.pkgname.split("."))-1]
        #优先处理icon，在写desktop文件
        self.install_desktop_file()
        file_content = self.generate_desktop_file(self.apkInfo.pkgname, self.apkInfo.displayname, self.apkInfo.pkgname, self.apkInfo.candidate_version)
        tmp_desktop_file = open(self.tmp_desktop_file_path, "w+")
        tmp_desktop_file.write(file_content)
        tmp_desktop_file.close()
        os.chmod(self.tmp_desktop_file_path, stat.S_IRWXU | stat.S_IRWXG | stat.S_IRWXO)
        # self.uninstall_desktop_file(pkgName = self.apkInfo.pkg_name)
        user_desktop_path = os.path.join(os.path.expanduser("~"), ".local", "share", "applications")
        if (os.path.exists(user_desktop_path) == False):
            os.mkdir(user_desktop_path)
        shutil.copy(self.tmp_desktop_file_path, user_desktop_path)  # 拷贝desktop文件

    # generate_desktop_file()函数用于生成desktop文件
    def generate_desktop_file(self, appName, appNameCN, pkgName, appVersion):
        home_path = os.path.expanduser('~')
        self.desktop_template = "[Desktop Entry]\n"\
                               + "Terminal=false\n"\
                               + "Type=Application\n"\
                               + "StartupNotify=false\n"\
                               + "Keywords=Android;App;Apk\n"\
                               + "Categories=Android;App;Apk\n"

        desktop_file = self.desktop_template
        desktop_file += "Name=" + appName
        desktop_file += "\nName[zh_CN]=" + appNameCN
        desktop_file += "\nComment=" + appName
        desktop_file += "\nComment[zh_CN]=" + appNameCN
        desktop_file += "\nExec=/usr/bin/startapp " + pkgName + " " + appVersion
        if(os.path.exists(home_path + "/.local/share/icons/" + pkgName + ".svg")):
            desktop_file += "\nIcon=" + home_path + "/.local/share/icons/" + pkgName + ".svg\n"
        elif(os.path.exists(home_path + "/.local/share/icons/" + pkgName + ".png")):
            desktop_file += "\nIcon=" + home_path + "/.local/share/icons/" + pkgName + ".png\n"

        return desktop_file

    # 将desktop文件转移到到用户.local/share/applications目录下
    def install_desktop_file(self):
        self.software_icon = UBUNTUKYLIN_RES_ICON_PATH + self.apkInfo.pkgname + ".svg"
        self.software_icon_cache = UBUNTUKYLIN_CACHE_ICON_PATH + self.apkInfo.pkgname + ".svg"
        self.software_icon_png = UBUNTUKYLIN_RES_ICON_PATH + self.apkInfo.pkgname + ".png"
        self.software_icon_cache_png = UBUNTUKYLIN_CACHE_ICON_PATH + self.apkInfo.pkgname + ".png"
        self.default_icon = UBUNTUKYLIN_RES_PATH + "default.png"
        icon_path = os.path.join(os.path.expanduser("~"), ".local", "share", "icons")
        icon_path_png = icon_path + "/" + self.apkInfo.pkgname + ".png"
        if(os.path.exists(icon_path) == False):
            os.mkdir(icon_path)

        if(os.path.exists(self.software_icon_cache)):
            # os.system("convert -resize 96x96 "+ self.software_icon_cache +" "+ UBUNTUKYLIN_CACHE_ICON_PATH +"ukui-android/96x96/apps/"+self.apkInfo.pkgname + ".png")
            # os.system("convert "+ UBUNTUKYLIN_CACHE_ICON_PATH +"ukui-android/96x96/apps/"+self.apkInfo.pkgname + ".png"+" "+UBUNTUKYLIN_CACHE_ICON_PATH+"ukui-android/scalable/apps/"+self.apkInfo.pkgname + ".svg")
            # png_to_svg(self.software_icon_cache,UBUNTUKYLIN_CACHE_ICON_PATH+"ukui-android/scalable/apps/")
            shutil.copy(self.software_icon_cache,icon_path)  # 拷贝缓存下的icons
            readline = os.popen("cat /usr/share/kylin-software-center/foot.txt").readline()
            os.system("sed -i '$i "+readline+ "' " +icon_path + "/" + self.apkInfo.pkgname + ".svg")
            # os.system("sed -i 's/[[:space:]]href/xlink\:href/' "+UBUNTUKYLIN_CACHE_ICON_PATH+"ukui-android/scalable/apps/"+self.apkInfo.pkgname + ".svg")
            os.system("sed -i 's/\"96\"/\"83\"/g' " + icon_path + "/" +self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/\"128\"/\"83\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/\"256\"/\"83\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/viewBox=\"0 0 128 128\"/viewBox=\"0 0 96 96\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/viewBox=\"0 0 256 256\"/viewBox=\"0 0 96 96\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
        elif(os.path.exists(self.software_icon)):
            shutil.copy(self.software_icon, icon_path)
            readline = os.popen("cat /usr/share/kylin-software-center/foot.txt").readline()
            os.system("sed -i '$i " + readline + "' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/\"96\"/\"83\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/\"128\"/\"83\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/\"256\"/\"83\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/viewBox=\"0 0 128 128\"/viewBox=\"0 0 96 96\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
            os.system(
                "sed -i 's/viewBox=\"0 0 256 256\"/viewBox=\"0 0 96 96\"/g' " + icon_path + "/" + self.apkInfo.pkgname + ".svg")
        elif(os.path.exists(self.software_icon_png)):
            shutil.copy(self.software_icon_png, icon_path)  # 拷贝缓存下的icons
        elif(os.path.exists(self.software_icon_cache_png)):
            shutil.copy(self.software_icon_png, icon_path)  # 拷贝缓存下的icons
        elif(os.path.exists(self.default_icon)):
            shutil.copy(self.default_icon, icon_path_png)

if __name__ == "__main__":
    download_sourcelist()



#!/usr/bin/python
# -*- coding: utf-8 -*-

### BEGIN LICENSE

# Copyright (C) 2013 National University of Defense Technology(NUDT) & Kylin Ltd

# Author:
#     Shine Huang<shenghuang@ubuntukylin.com>
# Maintainer:
#     Shine Huang<shenghuang@ubuntukylin.com>

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program.  If not, see <http://www.gnu.org/licenses/>.
from models.globals import Globals
import base64

#
# 函数：缓存读取
#
def password_read():
    try:
        file_object = open('backend/service/password.txt', 'r')
        # line = file_object.readlines()
        # line = line.strip()
        # print "aaaaaaaaaaaaaa",line

        line = file_object.readlines()
        if line != " ":

            Globals.SET_REM = base64.b64decode(line[0].strip()).decode()
            Globals.AUTO_LOGIN = base64.b64decode(line[1].strip()).decode()
            Globals.USER = base64.b64decode(line[2].strip()).decode()
            Globals.PASSWORD = base64.b64decode(line[3].strip()).decode()

        file_object.close()

    except:
        pass


#
# 函数：缓存写入
#
def password_write(set_rem_pass, auto_login, sequence, password):
    try:
        set_rem_pass = base64.b64encode(set_rem_pass.encode())
        auto_login = base64.b64encode(auto_login.encode())
        sequence = base64.b64encode(sequence.encode())
        password = base64.b64encode(password.encode())

        set_rem_pass = bytes.decode(set_rem_pass)
        auto_login = bytes.decode(auto_login)
        sequence = bytes.decode(sequence)
        password = bytes.decode(password)

        file_object = open('backend/service/password.txt', 'w')
        file_object.writelines(set_rem_pass + '\n')
        file_object.writelines(auto_login + '\n')
        file_object.writelines(sequence + '\n')
        file_object.writelines(password + '\n')
        file_object.close()
    except:
        pass






#!/usr/bin/python3
# -*- coding: utf-8 -*-

from PyQt5.QtWidgets import *
from PyQt5.QtCore import *
from PyQt5.QtGui import *

class Update_Source_Dialog(QMessageBox):

    def __init__(self, parent=None):
        QMessageBox.__init__(self, parent)
        #self.setWindowTitle("软件源更新提示")
        self.setWindowTitle(_("Software source update tips"))
        #self.checkbox = QCheckBox(self.tr("不再提醒"),self)
        self.checkbox = QCheckBox(self.tr(_("Never remind")), self)
        self.checkbox.isCheckable()

        self.checkbox.setGeometry(QRect(35, 120, 90, 20))
        #self.button_update = self.addButton(self.tr("更新"), QMessageBox.ActionRole)
        self.button_update = self.addButton(self.tr(_("Update")), QMessageBox.ActionRole)
        #self.button_notupdate = self.addButton(self.tr("不更新"), QMessageBox.ActionRole)
        self.button_notupdate = self.addButton(self.tr(_("Don't update")), QMessageBox.ActionRole)
        #self.button_exit = self.addButton(self.tr("退出"), QMessageBox.ActionRole)
        self.button_exit = self.addButton(self.tr(_("Quit")), QMessageBox.ActionRole)
        self.button_exit.hide()
        self.setEscapeButton(self.button_exit)


#!/usr/bin/python3
# -*- coding: utf-8 -*-

import os
import signal
import dbus
import dbus.mainloop.glib
from gi.repository import GObject
import sys
from run_utildbus import RunUtilDbus

if __name__ == '__main__':
    # os.environ["TERM"] = "xterm"
    os.environ["PATH"] = "/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin:/usr/games:/usr/local/games"
    # os.environ["DEBIAN_FRONTEND"] = "noninteractive"
    os.environ["QT_QPA_PLATFORMTHEME"] = "ukui"
    dbus.mainloop.glib.DBusGMainLoop(set_as_default=True)
    GObject.threads_init()
    mainloop = GObject.MainLoop()
    try:
        bus = dbus.SessionBus(mainloop)
    except:
        sys.exit(0)
    bus_name = dbus.service.BusName('com.kylin.softwarecenter.runapp', bus)
    dbusControler = RunUtilDbus(None, bus_name,mainloop)

    mainloop.run()